#include "player_states.hpp"
#include "player_character.hpp"
#include "tunnels_game_mode.hpp"
#include "utils/game_root.hpp"
#include <godot_cpp/variant/utility_functions.hpp>

namespace godot {
void PlayerState::_bind_methods() {
#define CLASSNAME PlayerState
}

void PlayerState::init(Node *target, StateMachine *machine) {
    this->state_machine = machine;
    this->target = Object::cast_to<PlayerCharacter>(target);
    if(!this->target)
        UtilityFunctions::push_error("Attempt to apply player state to non-PlayerCharacter node of type '", target->get_class(), "'");
    Ref<TunnelsGameMode> game_mode = GameRoot::get_singleton()->get_game_mode();
    this->player = game_mode->get_player_instance();
    if(!this->player)
        UtilityFunctions::push_error("Failed to find valid TunnelsPlayer instance in game mode");
}

StringName PlayerState::get_next() const {
    return this->get_class();
}

#undef CLASSNAME // PlayerState

void FollowingPlayer::_bind_methods() {
#define CLASSNAME FollowingPlayer
}

void FollowingPlayer::_process(double delta_time) {
    UtilityFunctions::print(this->get_path(), " _process");
    Vector3 const target_position = this->player->get_character()->get_global_position();
    if(target_position.distance_squared_to(this->target->get_global_position()) < 5.f * 5.f)
        this->target->move(this->player->get_character()->get_velocity().normalized());
    else
        this->target->move_to(target_position, 5.f);
}

#undef CLASSNAME // FollowingPlayer
}
